==> room-uikit-web-vue3-1.0.3.tgz: package/src/TUIRoom/extension/chatExtension.ts <==
import { ExtensionInfo, TUIConstants, TUICore, TUILogin } from '@tencentcloud/tui-core';
import TUIRoomEngine, {
  TUIErrorCode,
  TUIRole,
  TUIRoomEvents,
  TUIUserInfo,
} from '@tencentcloud/tuiroom-engine-js';
import { EventType, roomService } from '../services';
import { Profile } from '@tencentcloud/chat';
/* eslint-disable import/no-extraneous-dependencies, import/no-unresolved */
import interact from 'interactjs';


const roomOptions = {
  roomId: String(Math.ceil(Math.random() * 1000000)),
  roomMode: 'FreeToSpeak',
  roomParam: {
    isOpenCamera: true,
    isOpenMicrophone: true,
  },
};
interface MessagePayloadData {
  businessID: string
  groupId: string
  inviter: {
    faceUrl: string
    userId: string
    nickName: string
  }
  memberCount: number
  messageId: string
  owner: string
  ownerName: string
  roomId: string
  roomState: RoomState
  userList: Array<{ faceUrl: string; nickName: string; userId: string }>
  version: number
}
export interface MessagePayload {
  businessID: string
  data: MessagePayloadData
  extInfo: string
  platform: string
  room_id: string
  version: number
}
export enum RoomState {
  CREATING = 'creating',
  CREATED = 'created',
  DESTROYING = 'destroying',
  DESTROYED = 'destroyed'
}

export enum ChatType {
  C2C = 'c2c',
  GROUP = 'group',
  CUSTOM_SERVICE = 'customerService',
  ROOM = 'room'
}
// message 的编辑都交给房主处理，因此需要拿到每条message 然后对比 userid 相同的时候将其赋给message
export class ChatExtension {
  static instance?: ChatExtension;
  static message: any;
  static isInnerRoom: boolean;
  static chatContext: any;
  public myProfile = {} as Profile;
  static messagePayload: MessagePayload;

  private chatExtensionSetting: Record<ChatType, boolean> = {
    [ChatType.C2C]: true,
    [ChatType.GROUP]: true,
    [ChatType.CUSTOM_SERVICE]: false,
    [ChatType.ROOM]: false,
  };
  private TUIRoomKitMethods: any = null;
  constructor() {
    ChatExtension.instance = this;
  }
  static getInstance(): ChatExtension {
    if (!ChatExtension.instance) {
      ChatExtension.instance = new ChatExtension();
    }
    return ChatExtension.instance;
  }
  static destroyInstance(): void {
    if (!ChatExtension.instance) return;
    ChatExtension.instance = undefined;
  }
  public init() {
    this.TUIRoomKitMethods = roomService.TUIRoomKitMethods;
    this.bindRoomServiceEvent();
    TUICore.registerExtension(TUIConstants.TUIChat.EXTENSION.INPUT_MORE.EXT_ID, this);
    TUICore.registerEvent(
      TUIConstants.TUILogin.EVENT.LOGIN_STATE_CHANGED,
      TUIConstants.TUILogin.EVENT_SUB_KEY.USER_LOGIN_SUCCESS,
      this,
    );
    TUIRoomEngine.once('ready', () => {
      const roomEngine = new TUIRoomEngine();
      roomEngine?.callExperimentalAPI(JSON.stringify({
        api: 'setFramework',
        params: {
          component: 'TUIRoomKit',
          language: 'vue3',
        },
      }));
      this.bindRoomEngineEvent(roomEngine);
    });
  }

  public setChatExtension(chatType: ChatType, isShow: boolean) {
    this.chatExtensionSetting[chatType] = isShow;
  }
  private bindRoomServiceEvent() {
    roomService.on(EventType.ROOM_DESTROY, this.onRoomDestroy.bind(this));
  }
  private bindRoomEngineEvent(roomEngine: TUIRoomEngine) {
    roomEngine.on(TUIRoomEvents.onRemoteUserEnterRoom, this.onRemoteUserEnterRoom.bind(this));
    roomEngine.on(TUIRoomEvents.onRemoteUserLeaveRoom, this.onRemoteUserLeaveRoom.bind(this));
    roomEngine.on(TUIRoomEvents.onUserRoleChanged, this.onUserRoleChanged.bind(this));
  }
  private onRemoteUserEnterRoom({ userInfo }: { roomId: string; userInfo: TUIUserInfo }) {
    const { userID } = ChatExtension.chatContext;
    if (ChatExtension.messagePayload.data.owner !== userID) return;
    const { userList } = ChatExtension.messagePayload.data;
    const newUserList = [
      ...userList,
      {
        faceUrl: userInfo.avatarUrl,
        nickName: userInfo.userName,
        userId: userInfo.userId,
      },
    ];
    this.modifyMessage(ChatExtension.message?.ID, {
      userList: newUserList,
      memberCount: newUserList.length,
    });
  }
  private onRemoteUserLeaveRoom({ userInfo }: { roomId: string; userInfo: TUIUserInfo }) {
    const { userID } = ChatExtension.chatContext;
    if (ChatExtension.messagePayload.data.owner !== userID) return;
    const { userList } = ChatExtension.messagePayload.data;
    const newUserList = userList.filter(item => item.userId !== userInfo.userId);
    this.modifyMessage(ChatExtension.message?.ID, {
      userList: newUserList,
      memberCount: newUserList.length,
    });
  }
  private async onUserRoleChanged({ userId, userRole }: { userId: string; userRole: TUIRole }) {
    const { userID } = ChatExtension.chatContext;
    if (userRole === TUIRole.kRoomOwner && userId === userID) {
      const profileResult = await this.getUserProfile([userId]);
      const [profile] = profileResult.data;
      const { userID, nick } = profile;
      await this.modifyMessage(ChatExtension.message.ID, {
        owner: userID, // 房主的userId
        ownerName: nick, // 房主的userName
      });
    }
  }
  private onRoomDestroy() {
    this.destroyRoom();
  }
  public onGetExtension(extensionID: string, params: any) {
    const chatType:ChatType = params?.chatType;
    const extension: ExtensionInfo = {
      weight: 1000,
      text: '快速会议',
      icon: 'https://qcloudimg.tencent-cloud.cn/raw/148ab10dfe654076b41f0d0945bb82e8.png', // todo 待替换图标
      data: {
        name: 'quickRoom',
      },
      listener: {
        onClicked: (data: any) => {
          this.quickRoom(data);
        },
      },
    };
    if (!chatType) return extension; // 老版本 chatType === undefined 忽略配置直接 return
    if (!this.chatExtensionSetting[chatType]) return;
    return extension;
  }

  // 通过 onNotifyEvent 接收房间状态变化消息内部调用
  private async quickRoom(data: any) {
    const { conversationID, conversationType } = data;
    this.roomInit();
    const { chat, userID } = ChatExtension.chatContext;
    const message = this.createCustomMessage({
      chat,
      userID,
      roomId: roomOptions.roomId,
      conversationID,
      conversationType,
      roomState: RoomState.CREATING,
    });
    this.updateMessageList(message);
    await this.createRoom(roomOptions);
    ChatExtension.message = message;
    ChatExtension.messagePayload = this.parseMessageData(message);
    await this.sendMessage(message);
    await this.modifyMessage(message.ID, { messageId: message.ID, roomState: RoomState.CREATED });
    // todo 失败状态处理
  }
  private destroyRoom() {
    return this.modifyMessage(ChatExtension.message.ID, { roomState: RoomState.DESTROYED });
  }
  public onNotifyEvent(eventName: string, subKey: string) {
    if (eventName === TUIConstants.TUILogin.EVENT.LOGIN_STATE_CHANGED) {
      if (subKey === TUIConstants.TUILogin.EVENT_SUB_KEY.USER_LOGIN_SUCCESS) {
        // 收到登录成功时执行自己的业务逻辑处理
        ChatExtension.chatContext = TUILogin.getContext();
        chatExtension.myProfile = this.getMyProfile();
        this.roomInit();
      }
    }
  }

  private async roomInit() {
    const { SDKAppID, userID, userSig } = ChatExtension.chatContext;
    const { nick = '', avatar = '' } = this.myProfile;
    this.TUIRoomKitMethods.init({
      // 获取 sdkAppId 请您参考 步骤一
      sdkAppId: SDKAppID,
      // 用户在您业务中的唯一标示 Id
      userId: userID,
      // 本地开发调试可在 https://console.cloud.tencent.com/trtc/usersigtool 页面快速生成 userSig, 注意 userSig 与 userId 为一一对应关系
      userSig,
      // 用户在您业务中使用的昵称
      userName: nick,
      // 用户在您业务中使用的头像链接
      avatarUrl: avatar,
      // 用户在您业务中需要的皮肤主题颜色及是否支持切换皮肤主题
      theme: {
        defaultTheme: 'black',
        isSupportSwitchTheme: false,
      },
    });
  }
  public async createRoom(options: {
    roomId: string
    // roomId: String(Math.ceil(Math.random() * 1000000)),
    roomName?: string
    roomMode: string
    roomParam?: Record<string, any>
  }) {
    try {
      await this.TUIRoomKitMethods.createRoom(options);
    } catch (error: any) {
      alert(`TUIRoomKit.createRoom error: ${error.message}`);
    }
  }
  public async enterRoom(roomId: string, message: any) {
    ChatExtension.message = message;
    ChatExtension.messagePayload = this.parseMessageData(message);
    const roomParam = {
      isOpenCamera: true,
      isOpenMicrophone: true,
    };
    this.roomInit();
    try {
      await this.TUIRoomKitMethods.enterRoom({ roomId, roomParam });
    } catch (err: any) {
      if (err.code === TUIErrorCode.ERR_ROOM_ID_NOT_EXIST) {
        this.modifyMessage(ChatExtension.message.ID, { roomState: RoomState.DESTROYED });
      }
    }
  }
  private createCustomMessage(params: any) {
    const { chat, userID, conversationID, conversationType, roomState } = params;
    const { roomId } = roomOptions;
    const message = chat.createCustomMessage({
      to: conversationID.slice(conversationType.length),
      conversationType,
      payload: this.generatePayloadForMessage({
        roomId,
        userID,
        conversationID,
        roomState,
      }),
    });
    return message;
  }
  private async sendMessage(message: any) {
    const { chat } = ChatExtension.chatContext;
    const promise = chat.sendMessage(message);
    return promise;
  }
  private updateMessageList(message: any) {
    TUICore.callService({
      serviceName: TUIConstants.TUIChat.SERVICE.NAME,
      method: TUIConstants.TUIChat.SERVICE.METHOD.UPDATE_MESSAGE_LIST,
      params: {
        message,
      },
    });
  }
  private async modifyMessage(messageId: string, params: any) {
    const { chat } = ChatExtension.chatContext;
    const message = await chat.findMessage(messageId);
    const payloadData = this.parseMessageData(message);
    payloadData.data = {
      ...payloadData.data,
      ...params,
    };
    message.payload.data = JSON.stringify(payloadData);
    return await chat.modifyMessage(message);
  }
  private generatePayloadForMessage(params: {
    roomId: string
    userID: string
    conversationID: string
    roomState: RoomState
  }) {
    const { roomId, userID, conversationID, roomState } = params;
    const { nick = '', avatar = '' } = this.myProfile;
    const payload = {
      data: JSON.stringify({
        platform: 'Web',
        version: 1,
        businessID: 'ROOM_INVITE_ACTION', // 固定值，用于在IM上区分当前消息。
        room_id: roomId, // 房间Id, enterRoom 必须参数
        extInfo: '', // 预留字段
        data: {
          inviter: {
            avatarUrl: avatar,
            userId: userID,
            userName: nick,
          }, // 表明邀请人信息。用于在UI上展示 是谁邀请进会。
          version: 1,
          businessID: 'group_room_message',
          groupId: conversationID, // todo groupId需要拼起立
          messageId: '', // 用于观众成为房主后，通过messageId 来查找并更新指定消息。
          roomId, // 房间的id，enterRoom 必须的参数
          owner: userID, // 房主的userId
          ownerName: nick, // 房主的userName
          roomState, // 当前的房间状态，有creating/created/destroying/destroyed 四种状态
          memberCount: 1, // 当前房间内有多少人，需要在ui上展示有多少人在会议中。
          userList: [
            {
              faceUrl: avatar,
              nickName: nick,
              userId: userID,
            },
          ], // 包括房主在内的被邀请用户的列表，最多展示5个，防止消息长度超出限制。
        },
      }),
    };
    return payload;
  }

  public parseMessageData(message: any): MessagePayload {
    try {
      const data = message?.payload?.data;
      return data ? JSON.parse(data) : {};
    } catch (err) {
      console.warn('parseMessageData error:', err);
      return {} as MessagePayload;
    }
  }
  public getMyProfile() {
    const { chat } = ChatExtension.chatContext;
    return chat.getMyProfile();
  }

  private getUserProfile(userIDList: Array<string>) {
    const { chat } = ChatExtension.chatContext;
    return chat.getUserProfile({
      userIDList, // 请注意：即使只拉取一个用户的资料，也需要用数组类型，例如：userIDList: ['user1']
    });
  }
}

export const chatExtension = ChatExtension.getInstance();

interact('#roomContainer')
  .draggable({
    onmove: (event:any) => {
      const { target } = event;
      const x = (parseFloat(target.getAttribute('data-x')) || 0) + event.dx;
      const y = (parseFloat(target.getAttribute('data-y')) || 0) + event.dy;

      target.style.transform = `translate(${x}px, ${y}px)`;
      target.setAttribute('data-x', x);
      target.setAttribute('data-y', y);
    },
  })
  .resizable({
    edges: { top: true, left: true, bottom: true, right: true },
    listeners: {
      move(event) {
        let { x, y } = event.target.dataset;

        x = (parseFloat(x) || 0) + event.deltaRect.left;
        y = (parseFloat(y) || 0) + event.deltaRect.top;

        Object.assign(event.target.style, {
          width: `${event.rect.width}px`,
          height: `${event.rect.height}px`,
          transform: `translate(${x}px, ${y}px)`,
        });

        Object.assign(event.target.dataset, { x, y });
      },
    },
  })
  .styleCursor(true);

==> room-uikit-web-vue3-1.7.0.tgz: package/src/TUIRoom/extension/chatExtension.ts <==
import { ExtensionInfo, TUIConstants, TUICore, TUILogin } from '@tencentcloud/tui-core';
import {
  TUIErrorCode,
  TUIRole,
  TUIRoomEvents,
  TUIUserInfo,
} from '@tencentcloud/tuiroom-engine-js';
import { EventType, roomService, roomEngine } from '../services';
import { VueVersion } from './index';
import { setDragAndResize } from './utils/interact';
import { Conversation, Message, Profile } from '@tencentcloud/chat';
import { isMobile } from '../utils/useMediaValue';
import { setLanguage } from './utils/setLanguage';
import { provide } from 'vue';

const loadStyle = () => {
  import('./index.scss');
};

const roomOptions = {
  roomId: String(Math.ceil(Math.random() * 1000000)),
  roomMode: 'FreeToSpeak',
  roomParam: {
    isOpenCamera: true,
    isOpenMicrophone: true,
  },
};
export interface CustomMessagePayload {
  version: number
  businessID: string // 固定值，用于在IM上区分当前消息是哪类自定义消息。
  groupId: string // 邀请群成员入会时，需要用到groupId来获取群成员列表。
  messageId: string // 用于观众成为房主后，通过messageId 来查找并更新指定消息。
  roomId: string // 房间的id，enterRoom 必须的参数
  owner: string // 房主的userId
  ownerName: string // 房主的userName
  roomState: RoomState // 当前的房间状态，有creating/created/destroying/destroyed 四种状态
  memberCount: 1 // 当前房间内有多少人，需要在ui上展示有多少人在会议中。
  userList: Array<{ faceUrl: string; nickName: string; userId: string }> // 包括房主在内的被邀请用户的列表，最多展示5个，防止消息长度超出限制。
}
export enum RoomState {
  CREATING = 'creating',
  CREATED = 'created',
  DESTROYING = 'destroying',
  DESTROYED = 'destroyed'
}

export enum ChatType {
  C2C = 'c2c',
  GROUP = 'group',
  CUSTOM_SERVICE = 'customerService',
  ROOM = 'room'
}
// message 的编辑都交给房主处理，因此需要拿到每条message 然后对比 userid 相同的时候将其赋给message
export class ChatExtension {
  static instance?: ChatExtension;
  static message: Message;
  static isInnerRoom: boolean;
  static chatContext: {
    chat: Record<string, any>;
    SDKAppID: number;
    userID: string;
    userSig: string;
  };
  public myProfile = {} as Profile;
  static messagePayload: CustomMessagePayload;

  private chatExtensionSetting: Record<ChatType, boolean> = {
    [ChatType.C2C]: true,
    [ChatType.GROUP]: true,
    [ChatType.CUSTOM_SERVICE]: false,
    [ChatType.ROOM]: false,
  };
  private TUIRoomKitMethods: any = null;
  constructor() {
    ChatExtension.instance = this;
  }
  static getInstance(): ChatExtension {
    if (!ChatExtension.instance) {
      ChatExtension.instance = new ChatExtension();
    }
    return ChatExtension.instance;
  }
  static destroyInstance(): void {
    if (!ChatExtension.instance) return;
    ChatExtension.instance = undefined;
  }
  public init() {
    this.TUIRoomKitMethods = roomService.TUIRoomKitMethods;
    TUICore.registerExtension(TUIConstants.TUIChat.EXTENSION.INPUT_MORE.EXT_ID, this);
    TUICore.registerEvent(
      TUIConstants.TUILogin.EVENT.LOGIN_STATE_CHANGED,
      TUIConstants.TUILogin.EVENT_SUB_KEY.USER_LOGIN_SUCCESS,
      this,
    );
    TUICore.registerEvent(
      TUIConstants.TUITranslate.EVENT.LANGUAGE_CHANGED,
      TUIConstants.TUITranslate.EVENT_SUB_KEY.CHANGE_SUCCESS,
      this,
    );
  }

  public setChatExtension(chatType: ChatType, isShow: boolean) {
    this.chatExtensionSetting[chatType] = isShow;
  }
  private bindRoomServiceEvent() {
    roomService.on(EventType.ROOM_DESTROY, this.onRoomDestroy.bind(this));
  }
  private bindRoomEngineEvent() {
    roomEngine.instance?.on(TUIRoomEvents.onRemoteUserEnterRoom, this.onRemoteUserEnterRoom.bind(this));
    roomEngine.instance?.on(TUIRoomEvents.onRemoteUserLeaveRoom, this.onRemoteUserLeaveRoom.bind(this));
    roomEngine.instance?.on(TUIRoomEvents.onUserRoleChanged, this.onUserRoleChanged.bind(this));
  }
  private onRemoteUserEnterRoom({ userInfo }: { roomId: string; userInfo: TUIUserInfo }) {
    const { userID } = ChatExtension.chatContext;
    if (ChatExtension.messagePayload.owner !== userID) return;
    const { userList } = ChatExtension.messagePayload;
    const newUserList = [
      ...userList,
      {
        faceUrl: userInfo.avatarUrl,
        nickName: userInfo.userName,
        userId: userInfo.userId,
      },
    ];
    this.modifyMessage(ChatExtension.message?.ID, {
      userList: newUserList,
      memberCount: newUserList.length,
    });
  }
  private onRemoteUserLeaveRoom({ userInfo }: { roomId: string; userInfo: TUIUserInfo }) {
    const { userID } = ChatExtension.chatContext;
    if (ChatExtension.messagePayload.owner !== userID) return;
    const { userList } = ChatExtension.messagePayload;
    const newUserList = userList.filter(item => item.userId !== userInfo.userId);
    this.modifyMessage(ChatExtension.message?.ID, {
      userList: newUserList,
      memberCount: newUserList.length,
    });
  }
  private async onUserRoleChanged({ userId, userRole }: { userId: string; userRole: TUIRole }) {
    const { userID } = ChatExtension.chatContext;
    if (userRole === TUIRole.kRoomOwner && userId === userID) {
      const profileResult = await this.getUserProfile([userId]);
      const [profile] = profileResult.data;
      const { userID, nick } = profile;
      await this.modifyMessage(ChatExtension.message.ID, {
        owner: userID, // 房主的userId
        ownerName: nick, // 房主的userName
      });
    }
  }
  private onRoomDestroy() {
    this.destroyRoom();
  }
  public onGetExtension(extensionID: string, params: any) {
    const chatType: ChatType = params?.chatType;
    const extension: ExtensionInfo = {
      weight: 1000,
      text: '快速会议',
      icon: 'https://qcloudimg.tencent-cloud.cn/raw/148ab10dfe654076b41f0d0945bb82e8.png', // todo 待替换图标
      data: {
        name: 'quickRoom',
      },
      listener: {
        onClicked: (data: Conversation) => {
          this.quickRoom(data);
        },
      },
    };
    if (!chatType) return extension; // 老版本 chatType === undefined 忽略配置直接 return
    if (!this.chatExtensionSetting[chatType]) return;
    return extension;
  }

  // 通过 onNotifyEvent 接收房间状态变化消息内部调用
  private async quickRoom(data: Conversation) {
    if (ChatExtension.isInnerRoom) {
      alert('A user can only have one ongoing meeting at a time');
      return;
    }
    const { conversationID, type } = data;
    this.roomInit();
    const { chat, userID } = ChatExtension.chatContext;
    const message = this.createCustomMessage({
      chat,
      userID,
      roomId: roomOptions.roomId,
      conversationID,
      conversationType: type,
      roomState: RoomState.CREATING,
    });
    this.updateMessageList(message);
    await this.createRoom(roomOptions);
    ChatExtension.message = message;
    ChatExtension.messagePayload = this.parseMessageData(message);
    await this.sendMessage(message);
    await this.modifyMessage(message.ID, { messageId: message.ID, roomState: RoomState.CREATED });
  }
  private async destroyRoom() {
    await this.modifyMessage(ChatExtension.message.ID, { roomState: RoomState.DESTROYED });
    ChatExtension.isInnerRoom = false;
  }
  public onNotifyEvent(eventName: string, subKey: string, params?: Record<string, any>) {
    if (eventName === TUIConstants.TUILogin.EVENT.LOGIN_STATE_CHANGED) {
      if (subKey === TUIConstants.TUILogin.EVENT_SUB_KEY.USER_LOGIN_SUCCESS) {
        // 收到登录成功时执行自己的业务逻辑处理
        !isMobile && setDragAndResize('#roomContainer');
        roomEngine.instance?.callExperimentalAPI(JSON.stringify({
          api: 'setFramework',
          params: {
            component: 'TUIRoomKitFromChat',
            language: `vue${VueVersion}`,
          },
        }));
        this.bindRoomEngineEvent();
        this.bindRoomServiceEvent();
        loadStyle();
        provide('showHeaderInfo', false); // 隐藏退出登陆&&语言切换
        ChatExtension.chatContext = TUILogin.getContext();
        chatExtension.myProfile = this.getMyProfile();
        this.roomInit();
      }
    }
    if (eventName === TUIConstants.TUITranslate.EVENT.LANGUAGE_CHANGED) {
      if (subKey === TUIConstants.TUITranslate.EVENT_SUB_KEY.CHANGE_SUCCESS) {
        const language = params?.language;
        setLanguage(language);
      }
    }
  }

  private async roomInit() {
    this.TUIRoomKitMethods.getMediaDeviceList();
    const { SDKAppID, userID, userSig } = ChatExtension.chatContext;
    const { nick = '', avatar = '' } = this.myProfile;
    this.TUIRoomKitMethods.init({
      // 获取 sdkAppId 请您参考 步骤一
      sdkAppId: SDKAppID,
      // 用户在您业务中的唯一标示 Id
      userId: userID,
      // 本地开发调试可在 https://console.cloud.tencent.com/trtc/usersigtool 页面快速生成 userSig, 注意 userSig 与 userId 为一一对应关系
      userSig,
      // 用户在您业务中使用的昵称
      userName: nick,
      // 用户在您业务中使用的头像链接
      avatarUrl: avatar,
      // 用户在您业务中需要的皮肤主题颜色及是否支持切换皮肤主题
      theme: {
        defaultTheme: 'black',
        isSupportSwitchTheme: false,
      },
    });
  }
  public async createRoom(options: {
    roomId: string
    roomName?: string
    roomMode: string
    roomParam?: Record<string, any>
  }) {
    try {
      await this.TUIRoomKitMethods.createRoom(options);
    } catch (error: any) {
      alert(`TUIRoomKit.createRoom error: ${error.message}`);
    }
  }
  public async enterRoom(roomId: string, message: Message) {
    ChatExtension.message = message;
    ChatExtension.messagePayload = this.parseMessageData(message);
    const roomParam = {
      isOpenCamera: true,
      isOpenMicrophone: true,
    };
    this.roomInit();
    try {
      await this.TUIRoomKitMethods.enterRoom({ roomId, roomParam });
    } catch (err: any) {
      if (err.code === TUIErrorCode.ERR_ROOM_ID_NOT_EXIST) {
        this.modifyMessage(ChatExtension.message.ID, { roomState: RoomState.DESTROYED });
      }
    }
  }
  private createCustomMessage(params: Record<string, any>): Message {
    const { chat, userID, conversationID, conversationType, roomState } = params;
    const { roomId } = roomOptions;
    const message = chat.createCustomMessage({
      to: conversationID.slice(conversationType.length),
      conversationType,
      payload: this.generatePayloadForMessage({
        roomId,
        userID,
        conversationID,
        roomState,
      }),
    });
    return message;
  }
  private async sendMessage(message: Message) {
    const { chat } = ChatExtension.chatContext;
    return chat.sendMessage(message);
  }
  private updateMessageList(message: Message) {
    TUICore.callService({
      serviceName: TUIConstants.TUIChat.SERVICE.NAME,
      method: TUIConstants.TUIChat.SERVICE.METHOD.UPDATE_MESSAGE_LIST,
      params: {
        message,
      },
    });
  }
  private async modifyMessage(messageId: string, params: any) {
    const { chat } = ChatExtension.chatContext;
    const message = await chat.findMessage(messageId);
    let payloadData = this.parseMessageData(message);
    payloadData = {
      ...payloadData,
      ...params,
    };
    message.payload.data = JSON.stringify(payloadData);
    return await chat.modifyMessage(message);
  }
  private generatePayloadForMessage(params: {
    roomId: string
    userID: string
    conversationID: string
    roomState: RoomState
  }) {
    const { roomId, userID, conversationID, roomState } = params;
    const { nick = '', avatar = '' } = this.myProfile;
    const payload = {
      data: JSON.stringify({
        version: 1,
        businessID: 'group_room_message',
        groupId: conversationID, // todo groupId需要拼起立
        messageId: '', // 用于观众成为房主后，通过messageId 来查找并更新指定消息。
        roomId, // 房间的id，enterRoom 必须的参数
        owner: userID, // 房主的userId
        ownerName: nick, // 房主的userName
        roomState, // 当前的房间状态，有creating/created/destroying/destroyed 四种状态
        memberCount: 1, // 当前房间内有多少人，需要在ui上展示有多少人在会议中。
        userList: [
          {
            faceUrl: avatar,
            nickName: nick,
            userId: userID,
          },
        ], // 包括房主在内的被邀请用户的列表，最多展示5个，防止消息长度超出限制。
      }),
    };
    return payload;
  }

  public parseMessageData(message: Message): CustomMessagePayload {
    try {
      const data = message?.payload?.data;
      return data ? JSON.parse(data) : {};
    } catch (err) {
      console.warn('parseMessageData error:', err);
      return {} as CustomMessagePayload;
    }
  }
  public getMyProfile() {
    const { chat } = ChatExtension.chatContext;
    return chat.getMyProfile();
  }

  private getUserProfile(userIDList: Array<string>) {
    const { chat } = ChatExtension.chatContext;
    return chat.getUserProfile({
      userIDList, // 请注意：即使只拉取一个用户的资料，也需要用数组类型，例如：userIDList: ['user1']
    });
  }
}

setDragAndResize('#roomContainer');
export const chatExtension = ChatExtension.getInstance();
